#include <stdio.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include "gipsc.h"
#include "../../prips/prips.h"

#define DEFAULT_SAVE_FILE "ipsc.txt"

const char *TITLE = "IP Subnet Calculator";
const char *VERSION = "0.3.2";

struct network n;

struct MainControls mc;
struct SubnetInfo si;
struct GeneralInfo gi;
struct AddressInfo ai;
struct CidrInfo ci;

gint delete(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	gtk_main_quit();
	return FALSE;
}


void switch_page_cb(GtkWidget *widget, GtkNotebookPage *page, guint page_num, gpointer *data)
{

	if( GTK_LABEL(page->tab_label) == ci.tab_label)
	{
		gtk_label_set_text(mc.subnet_bits_label, "Supernet bits");
	}
	else
	{
		gtk_label_set_text(mc.subnet_bits_label, "Subnet bits");
	}
} 

/* When the OK button in the save dialog is clicked this function can
 * be called up to five times (there must be a better way!).  The first 
 * time it is called we find the file name where we are going to save.  
 * Subsequent calls check to see if the checkboxes we've added to the 
 * fselect main vbox are checked.  This function is called one additional
 * time for each check box.  If a particular check box is checked, we 
 * call the save routing for that check box.
 */
void save_data(GtkObject *obj)
{
	GtkLabel *label;
	gchar *label_text;
	static gchar *file_name;
	FILE *f;
	
	if( GTK_IS_FILE_SELECTION(obj))
	{
		file_name = gtk_file_selection_get_filename(
			GTK_FILE_SELECTION(obj)
		);
		return;
	}	
	else
	{
		if(!file_name)
			return;

		if((f = fopen((const char *) file_name, "a")) == NULL)
		{
			fprintf(stderr, "error opening %s\n", file_name);
			return;
		}

		if( GTK_TOGGLE_BUTTON(obj)->active)
		{
			label = GTK_LABEL( GTK_BIN(obj)->child);
			gtk_label_get(label, &label_text);
		
			if( strcmp(label_text, gi.tab_label->label) == 0)
			{
				fprint_general(f, &n);
			}
			else if( strcmp(label_text, si.tab_label->label) == 0)
			{
				fprint_subnets_hosts(f, &n);
			}
			else if( strcmp(label_text, ai.tab_label->label) == 0)
			{
				fprint_host(f, &n);
			}
			else if( strcmp(label_text, ci.tab_label->label) == 0)
			{
				fprint_cidr(f, &n);
			}
			else
			{
				g_assert_not_reached();
				return;	
			}
		}
		
		if(fclose(f) == EOF)
		{
			perror("fclose");
			return;
		}
	}
}

/* Create a file selection window.  Add our own functionality to
 * the selection's main vbox.  The user will be able to choose
 * what he/she'd like to save.
 */
void show_save_dialog()
{
	GtkFileSelection *fselect;
	GtkWidget *general_check, *subnets_check, *host_check, *cidr_check; 
	GtkWidget *vbox, *main;
	GtkWidget *frame;

	general_check = gtk_check_button_new_with_label(gi.tab_label->label);
	subnets_check = gtk_check_button_new_with_label(si.tab_label->label);
	host_check    = gtk_check_button_new_with_label(ai.tab_label->label);
	cidr_check    = gtk_check_button_new_with_label(ci.tab_label->label);

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(general_check), TRUE);
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(subnets_check), TRUE);
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(host_check), TRUE);
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cidr_check), TRUE);
 
	frame = gtk_frame_new("Selected Information");
 	vbox = gtk_vbox_new(FALSE, 0);

	fselect = GTK_FILE_SELECTION( gtk_file_selection_new("Save..."));
	main = fselect->main_vbox;
 
	gtk_widget_hide(fselect->ok_button);
	gtk_widget_hide(fselect->cancel_button);

	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", save_data, GTK_OBJECT(fselect));

	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", save_data, GTK_OBJECT(general_check));
	
	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", save_data, GTK_OBJECT(subnets_check));

	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", save_data, GTK_OBJECT(host_check));	

	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", save_data, GTK_OBJECT(cidr_check));

	gtk_signal_connect_object( GTK_OBJECT(fselect->ok_button),
		"clicked", gtk_widget_destroy, GTK_OBJECT(fselect));
        
	gtk_signal_connect_object( GTK_OBJECT(fselect->cancel_button),
		"clicked", gtk_widget_destroy, GTK_OBJECT(fselect));
         
	gtk_file_selection_set_filename(fselect, DEFAULT_SAVE_FILE);

	gtk_box_pack_start( GTK_BOX(main), frame, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), general_check, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), subnets_check, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), host_check, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), cidr_check, FALSE, FALSE, 0);      

	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
	gtk_widget_show_all(GTK_WIDGET(fselect));
}

void update_general_tab(const struct network *n)
{
	GString *subnet_max, *host_max, *netmask_hex, *subnet_mask_hex;
	int full_mask;
	
	subnet_max = g_string_new(NULL);
	host_max = g_string_new(NULL);
	netmask_hex = g_string_new(NULL);
	subnet_mask_hex = g_string_new(NULL);
		
	full_mask = n->mask | n->subnet_mask;
	
	g_string_sprintf(subnet_max, "%d", n->subnet_max);
	g_string_sprintf(host_max, "%d", n->host_max);
	g_string_sprintf(netmask_hex, "%X", n->mask);
	g_string_sprintf(subnet_mask_hex, "%X", full_mask);
	
 	gtk_entry_set_text( GTK_ENTRY(gi.subnet_max), subnet_max->str);
	gtk_entry_set_text( GTK_ENTRY(gi.host_max), host_max->str);
	gtk_entry_set_text( GTK_ENTRY(gi.bitmap), (gchar *) n->bitmap);
	gtk_entry_set_text( GTK_ENTRY(gi.subnet_mask), (gchar *) denumberize(full_mask)); 
	gtk_entry_set_text( GTK_ENTRY(gi.subnet_mask_hex), subnet_mask_hex->str);
	gtk_entry_set_text( GTK_ENTRY(gi.netmask), (gchar *) denumberize(n->mask));
	gtk_entry_set_text( GTK_ENTRY(gi.netmask_hex), netmask_hex->str);
	
	g_string_free(subnet_max, TRUE);
	g_string_free(host_max, TRUE);
	g_string_free(netmask_hex, TRUE);
	g_string_free(subnet_mask_hex, TRUE);
}

void update_subnets_hosts_tab(const struct network *n)
{
	unsigned long i, network = n->addr;
	GString *subnet_number, *host_range, *host_start, *host_end; 
	gchar *cols[2];
	
	subnet_number = g_string_new(NULL);
	host_range = g_string_new(NULL);
	host_start = g_string_new(NULL);
	host_end = g_string_new(NULL);
	
	gtk_clist_freeze( GTK_CLIST(si.subnet_list));
	gtk_clist_clear( GTK_CLIST(si.subnet_list));

	for(i = 0; i < n->subnet_max; i++)
	{
		g_string_sprintf(host_start, "%s", denumberize(network));
       		g_string_sprintf(host_end, "%s", denumberize((network + n->host_max) -1));
 
   		g_string_sprintf(subnet_number, "%ld", i + 1);
 		g_string_sprintf(host_range, "%s - %s", host_start->str, host_end->str);

		/* put and asteristk next to the current network */ 
		if(n->host_addr >= network && n->host_addr < (network + n->host_max))
                        g_string_append(host_range, " *");
	
 		cols[0] = subnet_number->str;
 		cols[1] = host_range->str;
  		
      		gtk_clist_append(GTK_CLIST(si.subnet_list), cols);

		network += n->host_max;
	}
	gtk_clist_thaw( GTK_CLIST(si.subnet_list));
}

void update_host_tab(const struct network *n)
{
	GString *addr_hex, *subnet_range;

	/*static int i;
	i++;
	g_print("update_host_tab: %d\n", i);*/

	addr_hex = g_string_new(NULL);
	subnet_range = g_string_new(NULL);

	g_string_sprintf(addr_hex, "%X", n->host_addr);
	g_string_sprintf(subnet_range, "%s - ", denumberize(n->host_network_lbound)); 
	g_string_append(subnet_range, (gchar *) denumberize(n->host_network_ubound));

	gtk_entry_set_text(ai.addr, (gchar *) denumberize(n->host_addr));
 	gtk_entry_set_text(ai.addr_hex, addr_hex->str);
	gtk_entry_set_text(ai.subnet_range, subnet_range->str);
	gtk_entry_set_text(ai.subnet_mask, (gchar *) denumberize(n->subnet_mask));
	gtk_entry_set_text(ai.subnet_id, (gchar *) 
		denumberize(n->subnet_mask & n->host_addr)); 
	gtk_entry_set_text(ai.network_id, (gchar *)
		denumberize(n->mask & n->host_addr));
	gtk_entry_set_text(ai.host_id, (gchar *) 
		denumberize(~(n->mask | n->subnet_mask) & n->host_addr));

	g_string_free(addr_hex, TRUE);
}

void update_cidr_tab(const struct network *n)
{
	gtk_entry_set_text(ci.cisco_wildcard, 
			   (gchar *)denumberize(n->cisco_wildcard));
}

/* Update the main sections of the interface.  We call set_network_info() 
 * here, so we must make sure that we update any other widgets after the 
 * call to update_tabs so that changes take effect in all of the widgets 
 * that we expect to be up to date.
 */
void update_tabs(struct network *n)
{
	/* update calculated data */
	set_network_info(n);

	/* update the interface */
	update_general_tab(n);
	update_subnets_hosts_tab(n);
	update_host_tab(n);
	update_cidr_tab(n);
}

void set_subnet_bits(GtkWidget *widget, GtkSpinButton *sbtn)
{
	n.subnet_bits = gtk_spin_button_get_value_as_int(sbtn);
	update_tabs(&n);
}

void set_interface_info(GtkWidget *widget, gpointer if_name)
{
	get_interface_info(&n, (char *) if_name);

	switch( tolower(n.class))
	{
	case 'a':
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(mc.class_a_button), 
					      TRUE);
		break;
	case 'b':
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(mc.class_b_button), 
					      TRUE);
		break;
	case 'c':	
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(mc.class_c_button), 
					      TRUE);
		break;
	}	
	
	update_tabs(&n);
	gtk_spin_button_set_value( GTK_SPIN_BUTTON(mc.subnet_bits), n.subnet_bits);
}

void set_class_info(GtkWidget *class_button, gpointer class)
{
	get_class_info(&n, GPOINTER_TO_INT(class), TRUE);
	update_tabs(&n);
	mc.subnet_bits_adj->upper = n.subnet_bits_max;
}

GtkWidget *get_main_menu()
{
	GtkWidget *menu;
	GtkWidget *menu_bar;
	GtkWidget *menu_item;
	GtkWidget *file_menu;
	GtkWidget *interface_menu;
	GtkWidget *handle_box;

	static char **if_names = NULL;
	int if_count, i;

	menu = gtk_menu_new();
	
	/* 
	 * File menu 
 	 */
	menu_item = gtk_menu_item_new_with_label("Save...");
 	gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
  	      GTK_SIGNAL_FUNC(show_save_dialog), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);

	menu_item = gtk_menu_item_new_with_label("Exit");
 	gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
  	      GTK_SIGNAL_FUNC(delete), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);

	file_menu = gtk_menu_item_new_with_label("File");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_menu), menu);

	/* 
	 * Interface menu 
	 */
	menu = gtk_menu_new();
	if_names = get_interface_names(&if_count);
	for(i = 0; i < if_count; i++)
	{
		menu_item = gtk_menu_item_new_with_label(if_names[i]);
		gtk_menu_append( GTK_MENU(menu), menu_item);
                gtk_signal_connect( GTK_OBJECT(menu_item), 
				    "activate",
                     		    GTK_SIGNAL_FUNC(set_interface_info), 
				    (gpointer) if_names[i]
		);
		gtk_widget_show(menu_item);
	}

	interface_menu = gtk_menu_item_new_with_label("Interfaces");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(interface_menu), menu);
	
	menu_bar = gtk_menu_bar_new();

	gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), file_menu);
	gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), interface_menu);
	
	handle_box = gtk_handle_box_new();
	
	gtk_container_add( GTK_CONTAINER(handle_box), menu_bar);

	gtk_widget_show_all(handle_box);

	return(handle_box);
}

void create_subnets_hosts_tab(void)
{
	gchar *titles[] = {"Subnet", "Host Range"};
	GtkScrolledWindow *scrolled_window;

	si.subnet_list = GTK_CLIST(gtk_clist_new_with_titles(2, titles));
	gtk_clist_set_column_width(si.subnet_list, 0, 60);

	scrolled_window = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new(NULL, NULL));
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrolled_window), 
					GTK_POLICY_AUTOMATIC, 
		 			GTK_POLICY_AUTOMATIC);

	si.tab_label = GTK_LABEL( gtk_label_new("Subnets/Hosts"));
	
	gtk_container_add( GTK_CONTAINER(scrolled_window), 
		GTK_WIDGET(si.subnet_list));

	gtk_widget_show( GTK_WIDGET(si.subnet_list));
	gtk_widget_show( GTK_WIDGET(scrolled_window));

	gtk_notebook_append_page( GTK_NOTEBOOK(mc.notebook), 
				  GTK_WIDGET(scrolled_window), 
				  GTK_WIDGET(si.tab_label));
}

void create_host_tab(void)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;
	
	table = gtk_table_new(4, 2, TRUE);
	gtk_container_border_width(GTK_CONTAINER(table), 5);

 	ai.addr 	= GTK_ENTRY( gtk_entry_new());
	ai.addr_hex	= GTK_ENTRY( gtk_entry_new());
	ai.subnet_range	= GTK_ENTRY( gtk_entry_new());
	ai.subnet_mask	= GTK_ENTRY( gtk_entry_new());
	ai.subnet_id	= GTK_ENTRY( gtk_entry_new());
	ai.network_id	= GTK_ENTRY( gtk_entry_new());
	ai.host_id	= GTK_ENTRY( gtk_entry_new());
	
	gtk_entry_set_editable(ai.addr, FALSE);
	gtk_entry_set_editable(ai.addr_hex, FALSE);
	gtk_entry_set_editable(ai.subnet_range, FALSE);
	gtk_entry_set_editable(ai.subnet_mask, FALSE);
	gtk_entry_set_editable(ai.subnet_id, FALSE);
	gtk_entry_set_editable(ai.network_id, FALSE);
	gtk_entry_set_editable(ai.host_id, FALSE);
			
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("IP address");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.addr),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 0, 1);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hexadecimal IP address");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.addr_hex),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 0, 1);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Address allocation range");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.subnet_range),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 2, 1, 2);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Subnet mask");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.subnet_mask), 
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 1, 2, 3);

 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Subnet ID");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.subnet_id),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 2, 3);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Network ID");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.network_id),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 1, 3, 4);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Host ID");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(ai.host_id),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 1, 2, 3, 4);

	gtk_widget_show_all(table);

	ai.tab_label = GTK_LABEL( gtk_label_new("Host"));
	gtk_notebook_append_page( GTK_NOTEBOOK(mc.notebook), 
				  GTK_WIDGET(table), 
				  GTK_WIDGET(ai.tab_label));
}

void create_cidr_tab(void)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;
	
	table = gtk_table_new(4, 2, TRUE);
	gtk_container_border_width(GTK_CONTAINER(table), 5);

 	ci.cisco_wildcard = GTK_ENTRY( gtk_entry_new());
	
	gtk_entry_set_editable(ci.cisco_wildcard, FALSE);
			
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Cisco wildcard");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(ci.cisco_wildcard),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 0, 1);
	
	gtk_widget_show_all(table);

	ci.tab_label = GTK_LABEL( gtk_label_new("CIDR"));
	gtk_notebook_append_page( GTK_NOTEBOOK(mc.notebook), 
				  GTK_WIDGET(table), 
				  GTK_WIDGET(ci.tab_label));
}

void create_general_tab(void)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;
	
	table = gtk_table_new(4, 2, TRUE);
	gtk_container_border_width(GTK_CONTAINER(table), 5);

 	gi.subnet_max 	= GTK_ENTRY( gtk_entry_new());
	gi.host_max 	= GTK_ENTRY( gtk_entry_new());
	gi.bitmap 	= GTK_ENTRY( gtk_entry_new());
	gi.netmask 	= GTK_ENTRY( gtk_entry_new());
	gi.netmask_hex 	= GTK_ENTRY( gtk_entry_new());
	gi.subnet_mask 	= GTK_ENTRY( gtk_entry_new());
	gi.subnet_mask_hex = GTK_ENTRY( gtk_entry_new());
	
	gtk_entry_set_editable(gi.subnet_max, FALSE);
	gtk_entry_set_editable(gi.host_max, FALSE);
	gtk_entry_set_editable(gi.bitmap, FALSE);
	gtk_entry_set_editable(gi.netmask, FALSE);
	gtk_entry_set_editable(gi.netmask_hex, FALSE);
	gtk_entry_set_editable(gi.subnet_mask, FALSE);
	gtk_entry_set_editable(gi.subnet_mask_hex, FALSE);
			
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Subnet maximum");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.subnet_max),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 0, 1);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hosts per subnet");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.host_max),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 0, 1);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Bitmap (n = network, s = subnet, h = host)");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.bitmap),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 2, 1, 2);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Class mask");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.netmask), 
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 1, 2, 3);

 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hexadecimal class mask");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.netmask_hex),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 1, 2, 2, 3);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Full subnet mask");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.subnet_mask),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 0, 1, 3, 4);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Full hexadecimal subnet mask");
	gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(gi.subnet_mask_hex),
		FALSE, FALSE, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), vbox, 1, 2, 3, 4);

	gtk_widget_show_all(table);

	gi.tab_label = GTK_LABEL( gtk_label_new("General"));
	gtk_notebook_append_page( GTK_NOTEBOOK(mc.notebook), 
				  GTK_WIDGET(table), 
				  GTK_WIDGET(gi.tab_label));
}

void create_tabs(void)
{
	create_general_tab();
	create_subnets_hosts_tab();
	create_host_tab();
	create_cidr_tab();
}

int main(int argc, char *argv[])
{
	GtkWindow *window;
	GtkWidget *frame;
	GtkWidget *vbox, *hbox, *vbox_any;
	GSList *group;

	gtk_init(&argc, &argv);

	window = GTK_WINDOW( gtk_window_new(GTK_WINDOW_TOPLEVEL));

	gtk_window_set_title(window, TITLE);
	
	gtk_signal_connect( GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete), NULL);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add( GTK_CONTAINER(window), vbox);

	gtk_box_pack_start( GTK_BOX(vbox), 
			    get_main_menu(), 
			    FALSE, 
			    FALSE, 
		            0
	);
	
	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start( GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	frame = gtk_frame_new("Address Type");
	gtk_container_border_width( GTK_CONTAINER(frame), 10);
	gtk_box_pack_start( GTK_BOX(hbox), frame, FALSE, FALSE, 0);

	vbox_any = gtk_vbox_new(FALSE, 5);
	gtk_container_border_width( GTK_CONTAINER(vbox_any), 5);
	gtk_container_add( GTK_CONTAINER(frame), vbox_any);

	/* 
	 * Create the class radio buttons 
	 */ 
	mc.class_a_button = gtk_radio_button_new_with_label(NULL, 
		"Class A (1.x.x.x - 126.x.x.x)");

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(mc.class_a_button));
	mc.class_b_button = gtk_radio_button_new_with_label(group, 
		"Class B (128.x.x.x - 191.x.x.x)");

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(mc.class_b_button));
	mc.class_c_button = gtk_radio_button_new_with_label(group, 
		"Class C (192.x.x.x - 223.x.x.x)");
	
	gtk_signal_connect(GTK_OBJECT(mc.class_a_button), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'a');

	gtk_signal_connect(GTK_OBJECT(mc.class_b_button), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'b');

	gtk_signal_connect(GTK_OBJECT(mc.class_c_button), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'c');

	gtk_box_pack_start(GTK_BOX(vbox_any), mc.class_a_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox_any), mc.class_b_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox_any), mc.class_c_button, FALSE, FALSE, 0);

	/*
	 * Controls to the right of the "Address Type" frame 
	 */
	vbox_any = gtk_vbox_new(FALSE, 5);
	gtk_container_border_width(GTK_CONTAINER(vbox_any), 10);
	gtk_box_pack_start( GTK_BOX(hbox), vbox_any, FALSE, FALSE, 0);

	/* 
	 * Subnet bits 
 	 */
	mc.subnet_bits_label = GTK_LABEL( gtk_label_new("Subnet bits"));
	gtk_box_pack_start(GTK_BOX(vbox_any), GTK_WIDGET(mc.subnet_bits_label),
		FALSE, FALSE, 0);
	
	mc.subnet_bits_adj = GTK_ADJUSTMENT( gtk_adjustment_new(0, 0, 31, 1, 1, 0));
	
	mc.subnet_bits = gtk_spin_button_new(mc.subnet_bits_adj, 0, 0);
 	
	gtk_signal_connect( GTK_OBJECT(mc.subnet_bits_adj), 
			    "value_changed",
			    GTK_SIGNAL_FUNC(set_subnet_bits),
			    (gpointer) mc.subnet_bits
	);
	gtk_box_pack_start( GTK_BOX(vbox_any), mc.subnet_bits, FALSE, FALSE, 0);

	mc.notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos( GTK_NOTEBOOK(mc.notebook), GTK_POS_TOP);
	gtk_box_pack_start( GTK_BOX(vbox), mc.notebook, FALSE, FALSE, 0);
	gtk_widget_set_usize(mc.notebook, 340, 300);

	create_tabs();
	
	gtk_signal_connect( GTK_OBJECT(mc.notebook),
			    "switch_page",
			    GTK_SIGNAL_FUNC(switch_page_cb),
			    NULL);

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(mc.class_c_button), TRUE);

	gtk_widget_show_all( GTK_WIDGET(window));

	gtk_main();
	
	return 0;
}
