#include <stdio.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include "gipsc.h"
#include "../../prips/prips.h"

#define DEFAULT_SAVE_FILE "ipsc.txt"

const char *TITLE = "IP Subnet Calculator";
const char *VERSION = "0.3";

static struct network n;
static struct subnet_info subnet_tab;
static struct general_info gi;

static GtkWidget *subnet_bits;
static GtkWidget *but_class_a, *but_class_b, *but_class_c;

static GtkAdjustment *subnet_bits_adj;

void delete(GtkWidget * widget, GtkWidget * event, gpointer data)
{
	gtk_main_quit ();
}

/* Create a file selection window.  Add our own functionality to
 * the selection's main vbox.  The user will be able to choose
 * what he/she'd like to save.
 */
void show_save_dialog()
{
	GtkFileSelection *fselect;
	GtkWidget *general_check, *subnets_check; 
	GtkWidget *vbox, *main;
	GtkWidget *frame;

	general_check = gtk_check_button_new_with_label("General");
	subnets_check = gtk_check_button_new_with_label("Subnets/Hosts");

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(general_check), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(subnets_check), TRUE);
 
	frame = gtk_frame_new("Selected information");
 	vbox = gtk_vbox_new(FALSE, 0);

	fselect = GTK_FILE_SELECTION(gtk_file_selection_new("Save..."));
	main = fselect->main_vbox;
 
	gtk_widget_hide(fselect->ok_button);
	gtk_widget_hide(fselect->cancel_button);

	/*gtk_signal_connect( GTK_OBJECT(fselect->ok_button),
		"clicked", (GtkSignalFunc) blah, fselect);*/
        
	gtk_signal_connect_object( GTK_OBJECT(fselect->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy,
		GTK_OBJECT(fselect));
         
	gtk_file_selection_set_filename(fselect, DEFAULT_SAVE_FILE);

	gtk_box_pack_start(GTK_BOX(main), frame, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), general_check, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), subnets_check, FALSE, FALSE, 0);
      
	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
	gtk_widget_show(general_check);
	gtk_widget_show(subnets_check); 
	gtk_widget_show(vbox);
	gtk_widget_show(frame);
	gtk_widget_show(fselect->ok_button); 
	gtk_widget_show(fselect->cancel_button); 
	gtk_widget_show(GTK_WIDGET(fselect));
}

void update_subnet_list(const struct network *n)
{
        unsigned long i, network = n->addr;
	GString *subnet_number, *host_range, *host_start, *host_end; 
	gchar *cols[2];
	
	subnet_number = g_string_new(NULL);
	host_range = g_string_new(NULL);
	host_start = g_string_new(NULL);
	host_end = g_string_new(NULL);
	
	gtk_clist_freeze(GTK_CLIST(subnet_tab.subnet_list));
	gtk_clist_clear(GTK_CLIST(subnet_tab.subnet_list));
        for(i = 0; i < n->subnet_max; i++)
        {
        	g_string_sprintf(host_start, "%s", denumberize(network));
        	g_string_sprintf(host_end, "%s", denumberize((network += n->host_max) -1));
 
   		g_string_sprintf(subnet_number, "%ld", i + 1);
 		g_string_sprintf(host_range, "%s - %s", host_start->str, host_end->str);
 		
 		cols[0] = subnet_number->str;
 		cols[1] = host_range->str;
  		
         	gtk_clist_append(GTK_CLIST(subnet_tab.subnet_list), cols);
        }
        gtk_clist_thaw(GTK_CLIST(subnet_tab.subnet_list));
}

void update_general_info(const struct network *n)
{
	GString *subnet_max, *host_max, *netmask_hex, *subnet_mask_hex;

	subnet_max = g_string_new(NULL);
	host_max = g_string_new(NULL);
	netmask_hex = g_string_new(NULL);
	subnet_mask_hex = g_string_new(NULL);
	
	g_string_sprintf(subnet_max, "%d", n->subnet_max);
	g_string_sprintf(host_max, "%d", n->host_max);
	g_string_sprintf(netmask_hex, "%X", n->mask);
	g_string_sprintf(subnet_mask_hex, "%X", n->subnet_mask);
	
 	gtk_entry_set_text(GTK_ENTRY(gi.subnet_max), subnet_max->str);
	gtk_entry_set_text(GTK_ENTRY(gi.host_max), host_max->str);
	gtk_entry_set_text(GTK_ENTRY(gi.bitmap), (gchar *) n->bitmap);
	gtk_entry_set_text(GTK_ENTRY(gi.subnet_mask), (gchar *) denumberize(n->subnet_mask));
	gtk_entry_set_text(GTK_ENTRY(gi.subnet_mask_hex), subnet_mask_hex->str);
	gtk_entry_set_text(GTK_ENTRY(gi.netmask), (gchar *) denumberize(n->mask));
	gtk_entry_set_text(GTK_ENTRY(gi.netmask_hex), netmask_hex->str);
	
	g_string_free(subnet_max, TRUE);
	g_string_free(host_max, TRUE);
	g_string_free(netmask_hex, TRUE);
	g_string_free(subnet_mask_hex, TRUE);
}

void set_subnet_bits(GtkWidget *widget, GtkSpinButton *sbtn)
{
	n.subnet_bits = gtk_spin_button_get_value_as_int(sbtn);
	set_network_info(&n);
	update_general_info(&n);
	update_subnet_list(&n);
}

void set_interface_info(GtkWidget *widget, gpointer if_name)
{
	get_interface_info(&n, (char *) if_name);
	set_network_info(&n);
	
	switch(tolower(n.class))
	{
	case 'a':
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(but_class_a), TRUE);
		break;
	case 'b':
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(but_class_b), TRUE);
		break;
	case 'c':	
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(but_class_c), TRUE);
		break;
	}	
	
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(subnet_bits), n.subnet_bits);
	update_general_info(&n);
	update_subnet_list(&n);
}

void set_class_info(GtkWidget *class_button, gpointer class)
{
	get_class_info(&n, (int) class);
	set_network_info(&n);
	subnet_bits_adj->upper = n.subnet_bits_max;
	update_general_info(&n);
	update_subnet_list(&n);
}

GtkWidget *get_main_menu()
{
	GtkWidget *menu;
	GtkWidget *menu_bar;
	GtkWidget *menu_item;
	GtkWidget *file_menu;
	GtkWidget *interface_menu;
	GtkWidget *handle_box;
	static char **if_names = NULL;
	int if_count, i;

	menu = gtk_menu_new();
	
	/* File menu */
	menu_item = gtk_menu_item_new_with_label("Save...");
 	gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
  	      GTK_SIGNAL_FUNC(show_save_dialog), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);

	menu_item = gtk_menu_item_new_with_label("Exit");
 	gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
  	      GTK_SIGNAL_FUNC(delete), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);

	file_menu = gtk_menu_item_new_with_label("File");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_menu), menu);
	gtk_widget_show(file_menu);

	/* Interface menu */
	menu = gtk_menu_new();
	if_names = get_interface_names(&if_count);
	for(i = 0; i <= if_count; i++)
	{
		menu_item = gtk_menu_item_new_with_label(if_names[i]);
		gtk_menu_append(GTK_MENU(menu), menu_item);
                gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
                     GTK_SIGNAL_FUNC(set_interface_info), (gpointer) if_names[i]);
		gtk_widget_show(menu_item);
	}

	interface_menu = gtk_menu_item_new_with_label("Interfaces");
	gtk_widget_show(interface_menu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(interface_menu), menu);
	
	menu_bar = gtk_menu_bar_new();
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), file_menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), interface_menu);
	
	handle_box = gtk_handle_box_new();

	gtk_widget_show(handle_box);
	gtk_widget_show(menu_bar);
	gtk_container_add(GTK_CONTAINER(handle_box), menu_bar);

	return(handle_box);
}

void create_subnet_tab(void)
{
	char *titles[] = {"Subnet", "Host Range"};
	GtkScrolledWindow *scrolled_window;

	subnet_tab.subnet_list = GTK_CLIST(gtk_clist_new_with_titles(2, titles));
	gtk_clist_set_column_width(subnet_tab.subnet_list, 0, 60);

	scrolled_window = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	subnet_tab.tab_label = gtk_label_new("Subnets/Hosts");
	
	gtk_container_add( GTK_CONTAINER(scrolled_window), 
		GTK_WIDGET(subnet_tab.subnet_list));

	gtk_widget_show( GTK_WIDGET(subnet_tab.subnet_list));
	gtk_widget_show( GTK_WIDGET(scrolled_window));

	gtk_notebook_append_page( GTK_NOTEBOOK(notebook), 
		GTK_WIDGET(scrolled_window), subnet_tab.tab_label);
}

void create_gi(void)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;
	
	table = gtk_table_new(4, 2, TRUE);
	gtk_container_border_width(GTK_CONTAINER(table), 5);

 	gi.subnet_max 	= GTK_ENTRY( gtk_entry_new());
	gi.host_max 	= GTK_ENTRY( gtk_entry_new());
	gi.bitmap 	= GTK_ENTRY( gtk_entry_new());
	gi.netmask 	= GTK_ENTRY( gtk_entry_new());
	gi.netmask_hex = GTK_ENTRY( gtk_entry_new());
	gi.subnet_mask = GTK_ENTRY( gtk_entry_new());
	gi.subnet_mask_hex = GTK_ENTRY( gtk_entry_new());
	
	gtk_entry_set_editable(gi.subnet_max, FALSE);
	gtk_entry_set_editable(gi.host_max, FALSE);
	gtk_entry_set_editable(gi.bitmap, FALSE);
	gtk_entry_set_editable(gi.netmask, FALSE);
	gtk_entry_set_editable(gi.netmask_hex, FALSE);
	gtk_entry_set_editable(gi.subnet_mask, FALSE);
	gtk_entry_set_editable(gi.subnet_mask_hex, FALSE);
			
	gtk_widget_show(GTK_WIDGET(gi.subnet_max));
	gtk_widget_show(GTK_WIDGET(gi.host_max));
	gtk_widget_show(GTK_WIDGET(gi.bitmap));
	gtk_widget_show(GTK_WIDGET(gi.netmask));
	gtk_widget_show(GTK_WIDGET(gi.netmask_hex));
	gtk_widget_show(GTK_WIDGET(gi.subnet_mask));
	gtk_widget_show(GTK_WIDGET(gi.subnet_mask_hex));
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Subnet maximum");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.subnet_max,
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 0, 1);
	gtk_widget_show(label);
	gtk_widget_show(vbox);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hosts per subnet");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.host_max,
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 0, 1);
	gtk_widget_show(label);
	gtk_widget_show(vbox);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Bitmap (n = network, s = subnet, h = host)");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.bitmap, FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 2, 1, 2);
	gtk_widget_show(label);
	gtk_widget_show(vbox);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Class mask");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.netmask, FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 2, 3);
	gtk_widget_show(label);
	gtk_widget_show(vbox);

 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hexadecimal class mask");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.netmask_hex,
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 2, 3);
	gtk_widget_show(vbox);
	gtk_widget_show(label);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Subnet mask");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.subnet_mask,
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 3, 4);
	gtk_widget_show(vbox);
	gtk_widget_show(label);
	
 	vbox = gtk_vbox_new(FALSE, 0);
	label = gtk_label_new("Hexadecimal subnet mask");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gi.subnet_mask_hex,
		FALSE, FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), vbox, 1, 2, 3, 4);
	gtk_widget_show(vbox);
	gtk_widget_show(label);

	gtk_widget_show(table);

	gi.tab_label = gtk_label_new("General");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, 
		gi.tab_label);
}

void create_pages(void)
{
	create_gi();
	create_subnet_tab();
}

int main(int argc, char *argv[])
{
	GtkWidget *window;
	GtkWidget *frame;
	GtkWidget *vbox, *hbox, *vbox_any;
	GtkWidget *menubar;
	GtkWidget *subnet_bits_label;
	GSList *group;

	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(GTK_WIDGET(window), 350, 450);
	gtk_window_set_title(GTK_WINDOW(window), TITLE);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(delete), NULL);

  	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_widget_show(vbox);

	menubar = get_main_menu();
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
	
	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	frame = gtk_frame_new("Address Type");
	gtk_container_border_width(GTK_CONTAINER(frame), 10);
	gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox_any = gtk_vbox_new(FALSE, 5);
	gtk_container_border_width(GTK_CONTAINER(vbox_any), 5);
	gtk_container_add( GTK_CONTAINER(frame), vbox_any);
	gtk_widget_show(vbox_any);
	
	 /* Create the class radio buttons */ 
	but_class_a = gtk_radio_button_new_with_label(NULL,
		"Class A (1.x.x.x - 126.x.x.x)");
	gtk_box_pack_start(GTK_BOX(vbox_any), but_class_a, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(but_class_a), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'a');
	gtk_widget_show(but_class_a);

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(but_class_a));
	but_class_b = gtk_radio_button_new_with_label(group,
		"Class B (128.x.x.x - 191.x.x.x)");
	gtk_box_pack_start(GTK_BOX(vbox_any), but_class_b, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(but_class_b), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'b');	
	gtk_widget_show(but_class_b);

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(but_class_b));
	but_class_c = gtk_radio_button_new_with_label(group, 
		"Class C (192.x.x.x - 223.x.x.x)");
	gtk_box_pack_start(GTK_BOX(vbox_any), but_class_c, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(but_class_c), "clicked", 
		GTK_SIGNAL_FUNC(set_class_info), (gpointer) 'c');	
	gtk_widget_show(but_class_c);

	/*  Controls to the right of the "Address Type" frame */
	vbox_any = gtk_vbox_new(FALSE, 5);
	gtk_container_border_width(GTK_CONTAINER(vbox_any), 10);
	gtk_box_pack_start(GTK_BOX(hbox), vbox_any, FALSE, FALSE, 0);
	gtk_widget_show(vbox_any);

	/* Subnet bits */
	subnet_bits_label = gtk_label_new("Subnet bits");
	gtk_box_pack_start(GTK_BOX(vbox_any), subnet_bits_label,
		FALSE, FALSE, 0);
	gtk_widget_show(subnet_bits_label);
	
	subnet_bits_adj = (GtkAdjustment *) gtk_adjustment_new(0, 0, 31, 1, 1, 0);
	subnet_bits = gtk_spin_button_new(subnet_bits_adj, 0, 0);
 	gtk_signal_connect(GTK_OBJECT(subnet_bits_adj), "value_changed",
			   GTK_SIGNAL_FUNC(set_subnet_bits),
			   (gpointer) subnet_bits);
	gtk_box_pack_start(GTK_BOX(vbox_any), subnet_bits, FALSE, FALSE, 0);
	gtk_widget_show(subnet_bits);

	notebook = gtk_notebook_new ();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
	gtk_box_pack_start (GTK_BOX (vbox), notebook, FALSE, FALSE, 0);
	gtk_widget_set_usize (notebook, 340, 300);
	gtk_widget_show(notebook);

	create_pages();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(but_class_c), TRUE);

	gtk_widget_show(vbox);
	gtk_widget_show(window);

	gtk_main();
	return 0;
}
