/* IP Subnet Calculator */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "../prips/prips.h"
#include "ipsc.h"

const char *MAINTAINER = "dan@vertekcorp.com";
const char *VERSION =
	"\ripsc 0.3.1
	\rThis program comes with NO WARRANTY, to the extent permitted 
	\rby law.  You may redistribute copies under the terms of the 
	\rGNU General Public License.\n";

struct network n;

void usage(const char *prog);

int main(int argc, char *argv[])
{
	char ch, *argstr = "c:r:i:sv";
	char *interface = "";
	
	/* flags */
	int print_subnet_flag = 0;
	int reverse_engineer_flag = 0;
	int class_flag = 0;

	opterr = 0;
        while ((ch = getopt(argc, argv, argstr)) != EOF) {
                switch (ch) {
                case 'c':
			if( get_class_info(&n, optarg[0], 0) < 0)
			{
				usage(argv[0]);
				exit(1);
			}
			class_flag = 1;
			break;
		case 'i':
		case 'r':
			reverse_engineer_flag = 1;
			interface = optarg;
			break;
		case 's':
			print_subnet_flag = 1;
			break;
		case 'v':
			printf("%s", VERSION);
			exit(0);
		case '?':
			usage(argv[0]);
			exit(1);
		}
	}

	if((optind - argc) == 0) 
	{
		if(reverse_engineer_flag)
		{
			if(get_interface_info(&n, interface) < 0) 
			{
				fprintf(stderr, 
					"%s: could not find interface \"%s\"\n",
					argv[0], interface);
				exit(1);
			}

		}
		else	
		{
			usage(argv[0]);
			exit(1);
		}
	}
	else
	{
		if(reverse_engineer_flag)
		{
			fprintf(stderr, "%s: too many interfaces\n", argv[0]);
			exit(1);
		}

		if(!class_flag)
		{
			fprintf(stderr, 
				"%s: no class specified, assuming class C\n" 
				"%s: to specify a class, use -c <class>\n", 
				argv[0], argv[0]);
			get_class_info(&n, 'c', 0);
		}
		n.subnet_bits = atoi(argv[optind]);
	}
	
	set_network_info(&n);
	if(incorrect_subnet_bits(&n))
	{
		fprintf(stderr, 
			"%s: too many subnet bits for network class\n", 
			argv[0]);
		exit(1);
	
	} 

	print_subnet_flag ? fprint_subnets_hosts(stdout, &n) : 
			    fprint_general(stdout, &n);
	return 0;
}

void usage(const char *prog)
{
        fprintf(stderr, "usage: %s [options] <subnet bits>
        -c <class>      Network class (a, b, or c)
        -r <if>		Reverse engineer and interface (e.g. eth0)
	-i		Same as -r
        -s 		Print all possible subnets
        -v		Print the program version
        -?		This help message

        \rReport bugs to %s\n",
                        prog, MAINTAINER);
}
